@@ -37,61 +37,103 @@ def hello_world(data):
 This will return "Hello World!" with a status code of 200, as plain text to the client
 """
 import importlib
+import mimetypes
 import re
 import urllib.parse as parse
 import warnings
 from functools import wraps
 from http.server import BaseHTTPRequestHandler, HTTPServer
+from typing import Union
 
+import serverly.stater
 from fileloghelper import Logger
 from serverly import default_sites
+from serverly.objects import Request, Response
 from serverly.utils import *
-import serverly.stater
 
-version = "0.1.5"
+version = "0.2.1"
 description = "A really simple-to-use HTTP-server"
 address = ("localhost", 8080)
-name = "PyServer"
+name = "Serverly"
 logger = Logger("serverly.log", "serverly", False, False)
 logger.header(True, True, description, fileloghelper_version=True,
               program_version="serverly v" + version)
 
 
 class Handler(BaseHTTPRequestHandler):
+
+    def respond(self, response: Response):
+        self.send_response(response.code)
+        for key, value in response.headers.items():
+            self.send_header(key, value)
+        self.end_headers()
+        self.wfile.write(bytes(response.body, "utf-8"))
+
     def do_GET(self):
         try:
             parsed_url = parse.urlparse(self.path)
-            response_code, content, info = _sitemap.get_content(
-                "GET", parsed_url.path)
+            data_length = int(self.headers.get("Content-Length", 0))
+            received_data = str(self.rfile.read(data_length), "utf-8")
+            request = Request("GET", parsed_url.path, dict(
+                self.headers), received_data, self.client_address)
+            response = _sitemap.get_content(request)
             logger.context = name + ": GET"
-            logger.debug(
-                f"Sent {str(response_code)}, path {parsed_url.path} (GET)")
-            print(response_code, info)
-            self.send_response(response_code)
-            for key in info:
-                self.send_header(key, info[key])
-                logger.debug(str(key) + "   " + str(info[key]))
-            self.end_headers()
-            self.wfile.write(bytes(content, "utf-8"))
+            logger.debug(str(response))
+            self.respond(response)
         except Exception as e:
-            serverly.stater.error()
+            serverly.stater.error(logger)
             logger.handle_exception(e)
             raise e
 
     def do_POST(self):
-        parsed_url = parse.urlparse(self.path)
-        length = int(self.headers.get("Content-Length", 0))
-        data = str(self.rfile.read(length), "utf-8")
-        response_code, content, info = _sitemap.get_content(
-            "POST", parsed_url.path, data)
-        logger.context = name + ": POST"
-        logger.debug(
-            f"Sent {str(response_code)}, path {parsed_url.path} (POST)")
-        self.send_response(response_code)
-        for key, value in info.items():
-            self.send_header(key, value)
-        self.end_headers()
-        self.wfile.write(bytes(content, "utf-8"))
+        try:
+            parsed_url = parse.urlparse(self.path)
+            data_length = int(self.headers.get("Content-Length", 0))
+            received_data = str(self.rfile.read(data_length), "utf-8")
+            request = Request("POST", parsed_url.path, dict(
+                self.headers), received_data, self.client_address)
+            response = _sitemap.get_content(request)
+            self.respond(response)
+            logger.context = name + ": POST"
+            logger.debug(str(response))
+        except Exception as e:
+            serverly.stater.error(logger)
+            logger.handle_exception(e)
+            raise e
+
+    def do_PUT(self):
+        try:
+            parsed_url = parse.urlparse(self.path)
+            data_length = int(self.headers.get("Content-Length", 0))
+            received_data = str(self.rfile.read(data_length), "utf-8")
+
+            response_code, content, headers = _sitemap.get_content(
+                "PUT", parsed_url.path, received_data)
+            self.respond(response_code, headers, content)
+            logger.context = name + ": PUT"
+            logger.debug(
+                f"Sent {str(response_code)}, path {parsed_url.path} (PUT)")
+        except Exception as e:
+            serverly.stater.error(logger)
+            logger.handle_exception(e)
+            raise e
+
+    def do_DELETE(self):
+        try:
+            parsed_url = parse.urlparse(self.path)
+            data_length = int(self.headers.get("Content-Length", 0))
+            received_data = str(self.rfile.read(data_length), "utf-8")
+
+            response_code, content, headers = _sitemap.get_content(
+                "DELETE", parsed_url.path, received_data)
+            self.respond(response_code, headers, content)
+            logger.context = name + ": DELETE"
+            logger.debug(
+                f"Sent {str(response_code)}, path {parsed_url.path} (DELETE)")
+        except Exception as e:
+            serverly.stater.error(logger)
+            logger.handle_exception(e)
+            raise e
 
 
 class Server:
@@ -206,11 +248,18 @@ class Sitemap:
         self.superpath = superpath
         self.methods = {
             "get": {},
-            "post": {}
+            "post": {},
+            "put": {},
+            "delete": {}
         }
         if error_page == None:
-            self.error_page = {0: StaticSite(
-                "/error", "none"), 404: default_sites.page_not_found_error, 500: default_sites.general_server_error}
+            self.error_page = {
+                0: StaticSite(
+                    "/error", "none"),
+                404: default_sites.page_not_found_error,
+                500: default_sites.general_server_error,
+                942: default_sites.user_function_did_not_return_response_object
+            }
         elif issubclass(error_page.__class__, StaticSite):
             self.error_page = {0: error_page}
         elif type(error_page) == dict:
@@ -264,93 +313,71 @@ class Sitemap:
             logger.warning(
                 f"Site for path '{path}' not found. Cannot be unregistered.")
 
-    def get_func_or_site_response(self, site, received_data: str):
-        response_code = 200
+    def get_func_or_site_response(self, site, request: Request):
+        response = Response()
         if isinstance(site, StaticSite):
             text = site.get_content()
-            info = {"Content-type": "text/html",
+            filetype = mimetypes.guess_type(site.file_path)
+            info = {"Content-type": filetype,
                     "Content-Length": len(text)}
         elif callable(site):
-            type_error_msg = f"Stuff that was returned by function {site.__name__} is not acceptable. Expected tuple[dict, str]."
             try:
-                content = site()
+                content = site(request)
             except TypeError:
                 try:
-                    try:
-                        data = json.loads(received_data)
-                    except json.JSONDecodeError:
-                        data = received_data
-                    content = site(data)
+                    content = site()
                 except TypeError as e:
                     logger.handle_exception(e)
                     raise TypeError(
-                        f"Function '{site.__name__}' either takes to many arguments (only data: str provided) or raises a TypeError")
+                        f"Function '{site.__name__}' either takes to many arguments (only object of type Request provided) or raises a TypeError")
                 except Exception as e:
                     logger.handle_exception(e)
-                    content = "500 - Internal server error"
+                    content = "500 - Internal server error - " + str(e)
                     raise e
             except Exception as e:
                 logger.handle_exception(e)
-                content = "500 - Internal server error"
+                content = "500 - Internal server error - " + str(e)
                 raise e
-            if type(content) == tuple:
-                v1 = False
-                v2 = False
-                if type(content[0]) == str:
-                    v1 = True
-                    if type(content[1]) == dict:
-                        response_code, info = parse_response_info(
-                            content[1], len(content[0]))
-                        text = content[0]
-                        v2 = True
-                    else:
-                        raise TypeError(type_error_msg)
-                if type(content[0]) == dict:
-                    if type(content[1]) == str:
-                        text = content[1]
-                        v2 = True
-                    else:
-                        raise TypeError(type_error_msg)
-                    response_code, info = parse_response_info(
-                        content[0], len(content[1]))
-                    v1 = True
-                if not v1 and not v2:
-                    raise ValueError(type_error_msg +
-                                     " Response was: " + str(content))
-            elif type(content) == str:
-                response_code, info = guess_response_info(content)
-                print(response_code, info)
-                text = content
-            elif type(content) == dict:
-                info = content
-                text = ""
-        text = text.replace(
+            response = Response(200, {}, "")
+            if isinstance(content, Response):
+                response = content
+            else:
+                try:
+                    raise UserWarning(
+                        f"Function for '{request.path}' needs to return a Response object. Website will be a warning message (not your content but serverly's).")
+                except Exception as e:
+                    logger.handle_exception(e)
+                response = self.get_func_or_site_response(
+                    self.error_page.get(942, self.error_page[0]), request)
+        response.body = response.body.replace(
             "/SUPERPATH/", self.superpath).replace("SUPERPATH/", self.superpath)
-        return response_code, info, text
+        return response
 
-    def get_content(self, method: str, path: str, received_data: str = ""):
+    def get_content(self, request: Request):
         response_code = 500
         text = ""
         info = {}
-        method = get_http_method_type(method)
-        check_relative_path(path)
         site = None
-        for pattern in self.methods[method]:
-            if re.match(pattern, path):
-                site = self.methods[method][pattern]
+        response = None
+        for pattern in self.methods[request.method].keys():
+            if re.match(pattern, request.path):
+                site = self.methods[request.method][pattern]
+                break
         if site == None:
             response_code = 404
             site = self.error_page.get(404, self.error_page[0])
+            response = self.get_func_or_site_response(
+                site, request)
         try:
-            response_code, info, text = self.get_func_or_site_response(
-                site, received_data)
+            response = self.get_func_or_site_response(
+                site, request)
         except Exception as e:
             logger.handle_exception(e)
             site = self.error_page.get(500, self.error_page[0])
-            response_code, info, text = self.get_func_or_site_response(
+            response = self.get_func_or_site_response(
                 site, "")
-            serverly.stater.error()
-        return response_code, text, info
+            serverly.stater.error(logger)
+        return response
 
 
 _sitemap = Sitemap()
@@ -404,6 +431,30 @@ def serves_post(path: str):
     return my_wrap
 
 
+def serves(method: str, path: str):
+    """When using this wrapper, please return a tuple with a dict containing 'response_code', and the content as a str.
+
+    Example(s):
+
+    return ({'response_code': 200}, 'Hello World!')
+
+    You can also give more headers:
+
+    return ({'response_code': 200, 'Content-type': 'text/plain', 'Content-Length': 4}, '1234')
+
+    Or, if you'd like to type as little as you can:
+
+    return {'code':200},'Hello there!'
+    """
+    def my_wrap(func):
+        _sitemap.register_site(method, func, path)
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            return func(*args, **kwargs)
+        return wrapper
+    return my_wrap
+
+
 def static_page(file_path, path):
     """Register a static page while the file is located under `file_path` and will serve `path`"""
     check_relative_file_path(file_path)
