@@ -20,6 +20,9 @@ def set(status_code: int):
         except st.ConnectionTimeout:
             raise Warning(
                 "Connection timeout to stater server. Cannot update status.")
+        except Exception as e:
+            raise Warning(
+                f"[{type(e).__name__}] Error while updating the status of the stater server: " + str(e))
 
 
 def setup(servername: str, serverpassword: str, componentname: str, errorthreshold=60):
@@ -46,20 +49,21 @@ def setup(servername: str, serverpassword: str, componentname: str, errorthresho
         raise TypeError("errorthreshold expected to be of type str.")
 
 
-def error():
+def error(logger):
     global _errors
     global error_treshold
-    print(_errors)
     try:
         now = datetime.datetime.now()
         _errors.append(now.isoformat())
         old = datetime.datetime.fromisoformat(_errors[-2])
         new = now - datetime.timedelta(minutes=error_threshold)
-        print(new, old)
         if new < old:
             stage = 2
         else:
             stage = 1
     except IndexError:
         stage = 1
-    set(stage)
+    try:
+        set(stage)
+    except Exception as e:
+        logger.handle_exception(e)
