@@ -15,14 +15,13 @@ def check_relative_path(path: str):
 def get_http_method_type(method: str):
     if type(method) != str:
         raise TypeError(
-            "method argument expected to be of type string. 'GET' and 'POST' are valid.")
-    if method.lower() == "get":
-        method = "get"
-    elif method.lower() == "post":
-        method = "post"
-    else:
+            "method argument expected to be of type string.")
+    supported_methods = ["get", "post", "put", "delete"  # , "head", "connect", "options", "trace", "patch"
+                         ]
+    method = method.lower()
+    if not method in supported_methods:
         raise Exception(
-            "Method argument invalid. Expected 'GET' or 'POST'.")
+            "Request method not supported. Supported are GET, POST, PUT & DELETE.")
     return method
 
 
@@ -56,13 +55,24 @@ def parse_response_info(info: dict, content_length=0):
     return response_code, info
 
 
+def guess_filetype_on_filename(filename):
+    file_types = {
+        "html": "text/html"
+    }
+    ext = filename.split(".")[-1]
+    return file_types.get(ext.lower(), "text/plain")
+
+
 def guess_response_info(content: str):
     if content.startswith("<!DOCTYPE html>") or content.startswith("<html"):
         c_type = "text/html"
     else:
         try:
-            json.loads(content)
+            if type(content) == str:
+                json.loads(content)
+            else:
+                json.dumps(content)
             c_type = "application/json"
         except json.JSONDecodeError:
             c_type = "text/plain"
-    return 200, {"Content-type": c_type, "Content-Length": len(content)}
+    return {"Content-type": c_type, "Content-Length": len(content)}
